/*******************************************************************************
* File Name: BOOTLDR.h  
* Version 2.20
*
* Description:
*  This file contains Pin function prototypes and register defines
*
* Note:
*
********************************************************************************
* Copyright 2008-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#if !defined(CY_PINS_BOOTLDR_H) /* Pins BOOTLDR_H */
#define CY_PINS_BOOTLDR_H

#include "cytypes.h"
#include "cyfitter.h"
#include "cypins.h"
#include "BOOTLDR_aliases.h"

/* APIs are not generated for P15[7:6] */
#if !(CY_PSOC5A &&\
	 BOOTLDR__PORT == 15 && ((BOOTLDR__MASK & 0xC0) != 0))


/***************************************
*        Function Prototypes             
***************************************/    

/**
* \addtogroup group_general
* @{
*/
void    BOOTLDR_Write(uint8 value);
void    BOOTLDR_SetDriveMode(uint8 mode);
uint8   BOOTLDR_ReadDataReg(void);
uint8   BOOTLDR_Read(void);
void    BOOTLDR_SetInterruptMode(uint16 position, uint16 mode);
uint8   BOOTLDR_ClearInterrupt(void);
/** @} general */

/***************************************
*           API Constants        
***************************************/
/**
* \addtogroup group_constants
* @{
*/
    /** \addtogroup driveMode Drive mode constants
     * \brief Constants to be passed as "mode" parameter in the BOOTLDR_SetDriveMode() function.
     *  @{
     */
        #define BOOTLDR_DM_ALG_HIZ         PIN_DM_ALG_HIZ
        #define BOOTLDR_DM_DIG_HIZ         PIN_DM_DIG_HIZ
        #define BOOTLDR_DM_RES_UP          PIN_DM_RES_UP
        #define BOOTLDR_DM_RES_DWN         PIN_DM_RES_DWN
        #define BOOTLDR_DM_OD_LO           PIN_DM_OD_LO
        #define BOOTLDR_DM_OD_HI           PIN_DM_OD_HI
        #define BOOTLDR_DM_STRONG          PIN_DM_STRONG
        #define BOOTLDR_DM_RES_UPDWN       PIN_DM_RES_UPDWN
    /** @} driveMode */
/** @} group_constants */
    
/* Digital Port Constants */
#define BOOTLDR_MASK               BOOTLDR__MASK
#define BOOTLDR_SHIFT              BOOTLDR__SHIFT
#define BOOTLDR_WIDTH              1u

/* Interrupt constants */
#if defined(BOOTLDR__INTSTAT)
/**
* \addtogroup group_constants
* @{
*/
    /** \addtogroup intrMode Interrupt constants
     * \brief Constants to be passed as "mode" parameter in BOOTLDR_SetInterruptMode() function.
     *  @{
     */
        #define BOOTLDR_INTR_NONE      (uint16)(0x0000u)
        #define BOOTLDR_INTR_RISING    (uint16)(0x0001u)
        #define BOOTLDR_INTR_FALLING   (uint16)(0x0002u)
        #define BOOTLDR_INTR_BOTH      (uint16)(0x0003u) 
    /** @} intrMode */
/** @} group_constants */

    #define BOOTLDR_INTR_MASK      (0x01u) 
#endif /* (BOOTLDR__INTSTAT) */


/***************************************
*             Registers        
***************************************/

/* Main Port Registers */
/* Pin State */
#define BOOTLDR_PS                     (* (reg8 *) BOOTLDR__PS)
/* Data Register */
#define BOOTLDR_DR                     (* (reg8 *) BOOTLDR__DR)
/* Port Number */
#define BOOTLDR_PRT_NUM                (* (reg8 *) BOOTLDR__PRT) 
/* Connect to Analog Globals */                                                  
#define BOOTLDR_AG                     (* (reg8 *) BOOTLDR__AG)                       
/* Analog MUX bux enable */
#define BOOTLDR_AMUX                   (* (reg8 *) BOOTLDR__AMUX) 
/* Bidirectional Enable */                                                        
#define BOOTLDR_BIE                    (* (reg8 *) BOOTLDR__BIE)
/* Bit-mask for Aliased Register Access */
#define BOOTLDR_BIT_MASK               (* (reg8 *) BOOTLDR__BIT_MASK)
/* Bypass Enable */
#define BOOTLDR_BYP                    (* (reg8 *) BOOTLDR__BYP)
/* Port wide control signals */                                                   
#define BOOTLDR_CTL                    (* (reg8 *) BOOTLDR__CTL)
/* Drive Modes */
#define BOOTLDR_DM0                    (* (reg8 *) BOOTLDR__DM0) 
#define BOOTLDR_DM1                    (* (reg8 *) BOOTLDR__DM1)
#define BOOTLDR_DM2                    (* (reg8 *) BOOTLDR__DM2) 
/* Input Buffer Disable Override */
#define BOOTLDR_INP_DIS                (* (reg8 *) BOOTLDR__INP_DIS)
/* LCD Common or Segment Drive */
#define BOOTLDR_LCD_COM_SEG            (* (reg8 *) BOOTLDR__LCD_COM_SEG)
/* Enable Segment LCD */
#define BOOTLDR_LCD_EN                 (* (reg8 *) BOOTLDR__LCD_EN)
/* Slew Rate Control */
#define BOOTLDR_SLW                    (* (reg8 *) BOOTLDR__SLW)

/* DSI Port Registers */
/* Global DSI Select Register */
#define BOOTLDR_PRTDSI__CAPS_SEL       (* (reg8 *) BOOTLDR__PRTDSI__CAPS_SEL) 
/* Double Sync Enable */
#define BOOTLDR_PRTDSI__DBL_SYNC_IN    (* (reg8 *) BOOTLDR__PRTDSI__DBL_SYNC_IN) 
/* Output Enable Select Drive Strength */
#define BOOTLDR_PRTDSI__OE_SEL0        (* (reg8 *) BOOTLDR__PRTDSI__OE_SEL0) 
#define BOOTLDR_PRTDSI__OE_SEL1        (* (reg8 *) BOOTLDR__PRTDSI__OE_SEL1) 
/* Port Pin Output Select Registers */
#define BOOTLDR_PRTDSI__OUT_SEL0       (* (reg8 *) BOOTLDR__PRTDSI__OUT_SEL0) 
#define BOOTLDR_PRTDSI__OUT_SEL1       (* (reg8 *) BOOTLDR__PRTDSI__OUT_SEL1) 
/* Sync Output Enable Registers */
#define BOOTLDR_PRTDSI__SYNC_OUT       (* (reg8 *) BOOTLDR__PRTDSI__SYNC_OUT) 

/* SIO registers */
#if defined(BOOTLDR__SIO_CFG)
    #define BOOTLDR_SIO_HYST_EN        (* (reg8 *) BOOTLDR__SIO_HYST_EN)
    #define BOOTLDR_SIO_REG_HIFREQ     (* (reg8 *) BOOTLDR__SIO_REG_HIFREQ)
    #define BOOTLDR_SIO_CFG            (* (reg8 *) BOOTLDR__SIO_CFG)
    #define BOOTLDR_SIO_DIFF           (* (reg8 *) BOOTLDR__SIO_DIFF)
#endif /* (BOOTLDR__SIO_CFG) */

/* Interrupt Registers */
#if defined(BOOTLDR__INTSTAT)
    #define BOOTLDR_INTSTAT            (* (reg8 *) BOOTLDR__INTSTAT)
    #define BOOTLDR_SNAP               (* (reg8 *) BOOTLDR__SNAP)
    
	#define BOOTLDR_0_INTTYPE_REG 		(* (reg8 *) BOOTLDR__0__INTTYPE)
#endif /* (BOOTLDR__INTSTAT) */

#endif /* CY_PSOC5A... */

#endif /*  CY_PINS_BOOTLDR_H */


/* [] END OF FILE */
